#ifndef _HISTOGRAM_BRUTEFORCE_HPP_
#define _HISTOGRAM_BRUTEFORCE_HPP_

#include "histodet/histogramAbstract.hpp"

namespace histodet {


/**
 * Computes histogram of a sliding window in an image with the brute force
 * OpenCV function.
 * See documentation of HistogramAbstract for interface description
 */
class HistogramBruteforce: public HistogramAbstract
{
 private:

 public:
	HistogramBruteforce(int hist_size, BucketerAbstract *bucketer);
	~HistogramBruteforce();
	void loadImage(cv::Mat const &image, int window_size);
	inline cv::Mat& get(cv::Rect const& rect);
	inline cv::Mat& getNext(int &pos_x, int &pos_y);
};



///*********************************************************************************************************************************************
/// Inlined functions
///*********************************************************************************************************************************************


cv::Mat& HistogramBruteforce::get(cv::Rect const& rect)
{
#if defined OPENCV
	const int channels[1] = {0};
	const float range[2] =  { 0, 256 };
	const float* ranges[1] = { range };
	int size[1] = {hist_size};
	cv::Mat win(img, rect);
	cv::calcHist(&win, 1, channels, cv::Mat(), hist(), 1, size, ranges, true, false);
#else
	hist.clear();
	for(int i = rect.y; i < rect.y+rect.height; ++i)
	for(int j = rect.x; j < rect.x+rect.width; ++j)
		hist.data()[*(img.data+img.step[0]*i+j)]++;
#endif
	return hist;
}


cv::Mat& HistogramBruteforce::getNext(int &pos_x, int &pos_y)
{
#ifndef NDEBUG
	if (!img.data) { std::cout << "Please load image before trying to compute histograms" << std::endl; return hist; }
#endif

	if (finished) { pos_x = pos_y = -1; return hist; }
	
	cv::Rect rect(cur_x-win_hsize, cur_y-win_hsize, win_size, win_size);
	get(rect);

	updatePos(pos_x, pos_y);
	return hist;
}


} // namespace histodet


#endif

