
#include <iostream>
#include "histodet/stretcher.hpp"

namespace histodet {



Stretcher::Stretcher(int hist_nbits, int min_support, bool normalize):
	hist_nbits(hist_nbits), hist_size(1 << hist_nbits),
	stretch_nbits(hist_nbits+2), stretch_size(1 << stretch_nbits),
	step_nbits(hist_nbits+stretch_nbits), min_support(min_support-1), normalize(normalize),
	hist_stretched(hist_size,1,CV_32F),
	hist_k(hist_size,1,CV_32F), hist_k1(hist_size,1,CV_32F),
	offset_k1(1 << hist_nbits), offset_x(2 << hist_nbits), offset_x1(3 << hist_nbits)
{
	if (posix_memalign(&lut, 32, hist_size * hist_size * stretch_size * sizeof(float)))
		{ std::cerr << "Aligned memory allocation failed!" << std::endl; }
	float j,x,x1; int k,k1;
	float norm = 0.0;

	for(int a = 0; a < hist_size; ++a)
	for(int b = a; b < hist_size; ++b)
	{
		if (normalize) norm = (b-a+1)/(float)hist_size;
		set_stretches(a, b);
		for(int i = 0; i < hist_size; ++i)
		{
			// a bit complex to avoid precision problems,
			// particularly ensure that if k1==hist_size then x1==0. because unitialized hist value
			j = a + i * (b-a) / (float)(hist_size-1);
			k = a + (i * (b-a)) / (hist_size-1);
			k1 = k+1;
			x1 = j-k;
			x = 1.-x1;
			if (x < 0.) { x = 0.; x1 = 1.; }
			if (k == hist_size-1 || x1 < 0.) { x = 1.; x1 = 0.; }
			if (normalize)
			{
				x *= norm;
				x1 *= norm;
			}
			
			stretch_k[i] = k;
			stretch_k1[i] = k1;
			stretch_x[i] = x;
			stretch_x1[i] = x1;
		}
	}
}

Stretcher::~Stretcher()
{
	free(lut);
}



} // namespace histodet



